<?php

namespace Nest;

use Nest\Exceptions\Services\Configuration\InvalidServiceConfigurationDefinitionException;
use Nest\Exceptions\Services\Configuration\UndefinedServiceConfigurationException;
use Nest\Services\ServiceConfiguration;
use function Nest\Utils\get_class_name_from_fullname;

/**
 * Main nest application class.
 */
abstract class Application
{
	/**
	 * Main instance of the application.
	 * @var static
	 */
	private static Application $application;

	/**
	 * Register the created application.
	 * @param static $application The application.
	 * @return static The application.
	 */
	public static function register(Application $application): static
	{
		self::$application = $application;
		return $application;
	}

	/**
	 * Get the main instance of the application.
	 * @return static The application.
	 */
	public static function get(): static
	{
		return self::$application;
	}


	/**
	 * Paths builder.
	 * @var Paths
	 */
	private Paths $paths;

	/**
	 * Services configurations associative array.
	 * @var array<string, ServiceConfiguration>
	 */
	private array $servicesConfigurations;


	/**
	 * Initialize a new application.
	 * @param string $basePath Base path of the application.
	 * @param string|null $globalContextIdentifier Global context identifier. This is useful if you need different configurations and initializations depending on call context.
	 * @throws UndefinedServiceConfigurationException
	 */
	public function __construct(string $basePath, private readonly ?string $globalContextIdentifier = null)
	{
		$this->paths = $this->initializePaths($basePath);
		$this->configure();
		$this->initializeServices();
		$this->initialize();
	}

	/**
	 * Get the global context identifier. This is useful if you need different configurations and initializations depending on call context.
	 * @return string|null The global context identifier.
	 */
	public function getGlobalContextIdentifier(): ?string
	{
		return $this->globalContextIdentifier;
	}


	/**
	 * Initialize application paths.
	 * @param string $basePath Base path of the application.
	 * @return Paths Paths builder.
	 */
	protected function initializePaths(string $basePath): Paths
	{
		// Create default paths builder.
		return new Paths($basePath);
	}

	/**
	 * Application paths.
	 * @return Paths
	 */
	public function paths(): Paths
	{
		return $this->paths;
	}


	/**
	 * Initialize application services.
	 * @return void
	 * @throws UndefinedServiceConfigurationException
	 */
	private function initializeServices(): void
	{
		foreach (class_uses($this) as $usedTrait)
		{
			// Build the service initializer name.
			$serviceInitializationMethod = "__nest__".get_class_name_from_fullname($usedTrait);
			if (method_exists($this, $serviceInitializationMethod))
				// If there is a service initializer, we run it.
				$this->$serviceInitializationMethod();
		}
	}

	/**
	 * Set a service configuration for a given base class.
	 * @param string $serviceConfigurationBaseClass The service configuration base class.
	 * @param ServiceConfiguration $serviceConfiguration Initialized service configuration instance.
	 * @return void
	 * @throws InvalidServiceConfigurationDefinitionException
	 */
	protected function setServiceConfiguration(string $serviceConfigurationBaseClass, ServiceConfiguration $serviceConfiguration): void
	{
		if (!is_a($serviceConfiguration, $serviceConfigurationBaseClass))
			throw new InvalidServiceConfigurationDefinitionException($serviceConfigurationBaseClass, get_class($serviceConfiguration));

		$this->servicesConfigurations[$serviceConfigurationBaseClass] = $serviceConfiguration;
	}

	/**
	 * Get a service configuration by its base class.
	 * @template T of ServiceConfiguration
	 * @param class-string<T> $serviceConfigurationClass Base class of the service configuration.
	 * @return T The service configuration instance.
	 * @throws UndefinedServiceConfigurationException
	 */
	protected function getServiceConfiguration(string $serviceConfigurationClass): ServiceConfiguration
	{
		if (empty($this->servicesConfigurations[$serviceConfigurationClass]))
			throw new UndefinedServiceConfigurationException($serviceConfigurationClass);

		/**
		 * Get service configuration.
		 * @var T $config
		 */
		$config = $this->servicesConfigurations[$serviceConfigurationClass];
		return $config;
	}

	/**
	 * Setup application configuration.
	 * Started before services initialization and application initialization.
	 * @return void
	 */
	protected abstract function configure(): void;


	/**
	 * Initialize application.
	 * Started after services initialization.
	 * @return void
	 */
	public function initialize(): void
	{}

	/**
	 * Exception handler instance.
	 * @var ExceptionHandler
	 */
	private ExceptionHandler $exceptionHandler;

	/**
	 * Get a new exception handler instance.
	 * @return ExceptionHandler New exception handler instance.
	 */
	protected function newExceptionHandler(): ExceptionHandler
	{
		// Return default exception handler.
		return new ExceptionHandler();
	}

	/**
	 * Get exception handler instance for this application.
	 * @return ExceptionHandler Exception handler instance.
	 */
	public function exceptionHandler(): ExceptionHandler
	{
		if (empty($this->exceptionHandler))
			// No exception handler, initializing one.
			$this->exceptionHandler = $this->newExceptionHandler();
		// Return initialized exception handler.
		return $this->exceptionHandler;
	}

	/**
	 * Start an engine in the application.
	 * @param callable $engine Engine instance or callable.
	 * @return void
	 */
	public function startEngine(callable $engine): void
	{
		// Set application exception handler while executing the engine.
		set_exception_handler($this->exceptionHandler());
		$engine();
		restore_exception_handler();
	}
}
