<?php

namespace Nest\Utils;

/**
 * Get first element of an array.
 * @template Element
 * @param Element[] $arr The array on which to get the first element.
 * @return Element|null First element of the array. NULL if array is empty.
 */
function array_first(array $arr): mixed
{
	// Try to get first element key.
	if (!is_null($firstKey = array_key_first($arr)))
		// There is a first element, returning it.
		return $arr[$firstKey];
	else
		// No first element, returning null.
		return null;
}

/**
 * Get first element of an array if it's one, or the direct value.
 * @template Element
 * @param Element|Element[] $arr_or_val Array or value.
 * @return Element Value or array first value.
 */
function array_first_or_val(mixed $arr_or_val): mixed
{
	return is_array($arr_or_val) ? array_first($arr_or_val) : $arr_or_val;
}

/**
 * Flatten arrays in one array.
 * @template Element
 * @param Element[] $items Array of arrays to flatten.
 * @return Element[] Flattened array.
 */
function array_flatten(array $items): array
{
	return array_reduce($items, function ($carry, $item) {
		return array_merge($carry, is_array($item) ? array_flatten($item) : [$item]);
	}, []);
}

/**
 * Deduplicate values in an array, but faster than array_unique.
 * Warning: duplicated values keys may be lost.
 *
 * @template Element
 * @param Element[] $array Array to deduplicate.
 * @return Element[] Deduplicated array.
 */
function array_unique_quick(array $array): array
{
	return array_flip(array_flip($array));
}

/**
 * Tests whether at least one element in the array passes the test implemented by the provided function.
 * It doesn't modify the array.
 * @template Element
 * @param callable $test_function - The function that implements the test.
 * @param Element[] $array - The array to test.
 * @return bool - True if it finds an element of the array for which the provided function returns true; returns false otherwise.
 */
function array_some(callable $test_function, array $array): bool
{
	foreach ($array as $key => $value)
	{ // For each (key => value) association, tests if the test function returns true.
		if ($test_function($value, $key))
			// If it returns true, we already can return true as we only search for one element that pass.
			return true;
	}

	return false; // We found no (key => value) association that passed the test, returning false.
}

