<?php

namespace Nest\Utils;

/**
 * Convert a camelCase string in a snake_case string.
 * @param string $camel_cased_str The camelCase string to convert.
 * @return string The converted string in snake_case.
 */
function str_camel_to_snake(string $camel_cased_str): string
{
	return ltrim(strtolower(preg_replace("/[A-Z]([A-Z](?![a-z]))*/", "_$0", $camel_cased_str)), "_");
}

/**
 * Get the plural form of every word in the snake_cased string.
 * @param string $snake_name_to_pluralize The snake_cased string to pluralize.
 * @return string The pluralized snake_cased string.
 */
function str_snake_pluralize(string $snake_name_to_pluralize): string
{
	// Break the string to an array containing each word.
	$parts = explode("_", $snake_name_to_pluralize);
	// Add an 's' to each word.
	$parts = array_map(fn (string $part) => "{$part}s", $parts);
	// Glue all words together with '_'.
	return implode("_", $parts);
}

/**
 * Get the singular form of every word in the snake_cased string.
 * @param string $snake_name_to_singularize The snake_cased string to singularize.
 * @return string The singularized snake_cased string.
 */
function str_snake_singularize(string $snake_name_to_singularize): string
{
	// Break the string to an array containing each word.
	$parts = explode("_", $snake_name_to_singularize);
	// Remove an 's' from each word, if there is one.
	$parts = array_map(function (string $part) {
		$sPos = strlen($part) - 1;
		return $part[$sPos] == 's' ? substr($part, 0, $sPos) : $part;
	}, $parts);
	// Glue all words together with '_'.
	return implode("_", $parts);
}

/**
 * Remove prefixes from a base string, if present.
 * @param string $base Base string.
 * @param string[] $prefix Prefix to remove, if present.
 * @return string
 */
function str_remove_prefix(string $base, string ...$prefix): string
{
	foreach($prefix as $_prefix)
	{ // Remove each passed prefix.
		if (str_starts_with($base, $_prefix))
			// Prefix is present, removing it.
			$base = substr($base, strlen($_prefix));
	}

	return $base; // Return altered base.
}

/**
 * Remove suffixes from a base string, if present.
 * @param string $base Base string.
 * @param string[] $suffix Suffix to remove, if present.
 * @return string
 */
function str_remove_suffix(string $base, string ...$suffix): string
{
	foreach($suffix as $_suffix)
	{ // Remove each passed suffix.
		if (str_ends_with($base, $_suffix))
			// Suffix is present, removing it.
			$base = substr($base, 0, strlen($base) - strlen($_suffix));
	}

	return $base; // Return altered base.
}
